tsoutliers2 = function (x, 
                        lowFenceDistIQR = 3, 
                        highFenceDistIQR = 3,
                        iterate = 2, 
                        lambda = NULL) 
{
    n <- length(x)
    freq <- frequency(x)
    missng <- is.na(x)
    nmiss <- sum(missng)
    if (nmiss > 0L) {
        xx <- na.interp(x, lambda = lambda)
    }
    else {
        xx <- x
    }
    if (is.constant(xx)) {
        return(list(index = integer(0), replacements = numeric(0)))
    }
    if (!is.null(lambda)) {
        xx <- BoxCox(xx, lambda = lambda)
        lambda <- attr(xx, "lambda")
    }
    if (freq > 1 && n > 2 * freq) {
        fit <- mstl(xx, robust = TRUE)
        rem <- remainder(fit)
        detrend <- xx - trendcycle(fit)
        strength <- 1 - var(rem)/var(detrend)
        if (strength >= 0.6) {
            xx <- seasadj(fit)
        }
    }
    tt <- 1:n
    mod <- supsmu(tt, xx)
    resid <- xx - mod$y
    if (nmiss > 0L) {
        resid[missng] <- NA
    }
    resid.q <- quantile(resid, probs = c(0.25, 0.75), na.rm = TRUE)
    iqr <- diff(resid.q)
    limits <- resid.q + iqr * c(-lowFenceDistIQR, highFenceDistIQR)
    if ((limits[2] - limits[1]) > 1e-14) {
        outliers <- which((resid < limits[1]) | (resid > limits[2]))
    }
    else {
        outliers <- numeric(0)
    }
    x[outliers] <- NA
    x <- na.interp(x, lambda = lambda)
    if (iterate > 1) {
        tmp <- tsoutliers2(x, 
                           lowFenceDistIQR = lowFenceDistIQR, 
                           highFenceDistIQR = highFenceDistIQR,
                           iterate = 1, lambda = lambda)
        if (length(tmp$index) > 0) {
            outliers <- sort(unique(c(outliers, tmp$index)))
            x[outliers] <- NA
            x <- na.interp(x, lambda = lambda)
        }
    }
    return(list(index = outliers, replacements = x[outliers]))
}